! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

module time_mod

  use param
  use astrolib
  use iso_fortran_env

  implicit none

  private
  public :: sec2date

  contains

!--------------------------------------------------------------
  subroutine sec2date (tsec, mjdref, dateobs, ierr)
!--------------------------------------------------------------

! converts spacecraft clock to date-obs date and time string,
! uses routine JDCAL1  from astrolib for JD to date conversion,
! handling of leap seconds added by HB (works as long as MJDREF
! isn't before 1999)

  real(real64), intent(in) :: tsec, mjdref
  character(len=19), intent(out) :: dateobs

! *** current till end of 2018 ***

! Leap seconds since 2000 - day AFTER leap second was inserted:
! 1: 01-JAN-2006
! 2: 01-JAN-2009
! 3: 01-JUL-2012
! 4: 01-JUL-2015
! 5: 01-JAN-2017 

 integer, parameter :: nls = 5
! MJD of day AFTER a leap second was inserted
  integer, dimension(nls), parameter :: leapdate = &
     (/ 53736, 54832, 56109, 57204, 57754 /) 

  real(real64), parameter :: mjdmin = 51179d0

  real(real64) :: mjd, jd
  integer :: i, yr, mm, dd, hr, mt, ierr 
  real :: sec

  real(real64), parameter :: leapsec = 1d0/86400d0

  if (mjdref .lt. mjdmin) then
    ierr=3 
    call wrerr ("sec2date", ierr, "MJDREF is outside allowed range")
    return
  end if

  mjd = tsec/86400d0 + mjdref 

  do i=1,nls
    if (mjd .ge. dble(leapdate(i))) mjd = mjd - leapsec
  end do  

  jd = mjd + 2400000.5d0

  call jdcal1 (jd, yr, mm, dd, hr, mt, sec, ierr)

  write (dateobs, '(i4.4,2(''-'',i2.2),''T'',i2.2,2('':'',i2.2))') &
                    yr,mm,dd,hr,mt,nint(sec)

end subroutine sec2date

end module time_mod
